/*****************************************************************************/
/* EyeClock - A clock application for X. The eyes follow the mouse.          */
/*                                                                           */
/* EyeClock Copyright (c) 1998 Sakai Hiroaki.                                */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* BSD Daemon Copyright 1988 by Marshall Kirk McKusick.                      */
/* All Rights Reserved.                                                      */
/*                                                                           */
/* EyeClock is a simple clock application for X11. You can put an optional   */
/* picture on the face of the clock. And you can move the eyes of the        */
/* picture following the mouse.                                              */
/*                                                                           */
/* EyeClock is a free software. You may copy or distribute the original      */
/* EyeClock freely. But, you may not modify or distribute the original       */
/* EyeClock without permission of the author.                                */
/*                                                                           */
/* Statement on the Use of the BSD Daemon Figure:                            */
/*                                                                           */
/* The BSD Daemon is to be used in the context of BSD software.              */
/* So, if you are using BSD software (BSD/OS, FreeBSD, NetBSD, OpenBSD,      */
/* or BSD utilities) in your Intranet environment, then use of the           */
/* daemon is appropriate. If you are a Microsoft shop, then it is not        */
/* appropriate.                                                              */
/*                                                                           */
/* Individuals may use the daemon for their personal use within the          */
/* bounds of good taste (an example of bad taste was a picture of the        */
/* BSD daemon blowtorching a Solaris logo). When reasonably possible,        */
/* I would like the text ``BSD Daemon Copyright 1988 by Marshall Kirk        */
/* McKusick. All Rights Reserved.'' to be included.  This text need          */
/* not be etched into the figure or garishly displayed when using the        */
/* daemon as say an Icon in a Web window. A good example of how              */
/* to handle the due credit in a web page is to create a link from           */
/* the daemon picture to the following text:                                 */
/*                                                                           */
/*         BSD Daemon Copyright 1988 by Marshall Kirk McKusick.              */
/*         All Rights Reserved.                                              */
/*                                                                           */
/*         Permission to use the daemon may be obtained from:                */
/*                 Marshall Kirk McKusick                                    */
/*                 1614 Oxford St                                            */
/*                 Berkeley, CA 94709-1608                                   */
/*                 USA                                                       */
/*         or via email at mckusick@mckusick.com                             */
/*                                                                           */
/* If you are looking for daemon images, a pictorial history of the          */
/* daemon can be found at http://www.mckusick.com/beastie/index.html.        */
/*                                                                           */
/* If you want to mass produce the daemon on Tshirts, CDROM's, or other      */
/* products you need to request permission in advance. In general, I require */
/* that the daemon be used in an appropriate way. This means that it         */
/* has to be something related to BSD and not expropriated as a company      */
/* logo (though I do allow companies with BSD-based products such as         */
/* Walnut Creek CDROM or BSDI to use it).  I regret having to be so          */
/* legalistic about the daemon, but I almost lost the daemon to a            */
/* certain large company because I failed to show due diligence in           */
/* protecting it. So, I've taken due diligence seriously since then.         */
/*                                                                           */
/*         Marshall Kirk McKusick                                            */
/*****************************************************************************/

/*****************************************************************************/
/* إåΥ󥯥롼                                                      */
/*****************************************************************************/

#include"eyetype.h"

/*****************************************************************************/
/* ǥեͤ                                                        */
/*****************************************************************************/

void SetDefaultValue(Clock * cp, int argc, char ** argv)
{
  *(cp->pixmap_filename) = '\0';
  strcpy(cp->clock_name, DEFAULT_CLOCK_NAME);
  cp->width           = DEFAULT_WIDTH;
  cp->height          = DEFAULT_HEIGHT;
  cp->clock_flag      = DEFAULT_CLOCK_FLAG;
  cp->eye_flag        = DEFAULT_EYE_FLAG;
  cp->every_flag      = DEFAULT_EVERY_FLAG;
  cp->verbose_flag    = DEFAULT_VERBOSE_FLAG;
  strcpy(cp->default_eye_foreground_color,   DEFAULT_EYE_FOREGROUND_COLOR);
  strcpy(cp->default_eye_background_color,   DEFAULT_EYE_BACKGROUND_COLOR);
  strcpy(       cp->long_foreground_color,  DEFAULT_LONG_FOREGROUND_COLOR);
  strcpy(       cp->long_background_color,  DEFAULT_LONG_BACKGROUND_COLOR);
  strcpy(      cp->short_foreground_color, DEFAULT_SHORT_FOREGROUND_COLOR);
  strcpy(      cp->short_background_color, DEFAULT_SHORT_BACKGROUND_COLOR);
  cp->long_length = DEFAULT_LONG_LENGTH;
  cp->long_foreground_thickness = DEFAULT_LONG_FOREGROUND_THICKNESS;
  cp->long_background_thickness = DEFAULT_LONG_BACKGROUND_THICKNESS;
  cp->short_length = DEFAULT_SHORT_LENGTH;
  cp->short_foreground_thickness = DEFAULT_SHORT_FOREGROUND_THICKNESS;
  cp->short_background_thickness = DEFAULT_SHORT_BACKGROUND_THICKNESS;
  cp->clock_center.x = DEFAULT_CLOCK_CENTER_X;
  cp->clock_center.y = DEFAULT_CLOCK_CENTER_Y;
  cp->animation_flag = OFF;
  cp->animation_type = 0;

  if (argc == 1) {
    cp->read_data_from = READ_FROM_STRING;
    cp->read_pixmap_from = READ_FROM_STRING;
    cp->data_filename = NULL;
  } else if (!strcmp(argv[1], "-pixmap")) {
    cp->read_data_from = NOT_READ;
    cp->read_pixmap_from = READ_FROM_FILE;
    cp->data_filename = NULL;
  } else if (*(argv[1]) != '-') {
    cp->read_data_from = READ_FROM_FILE;
    cp->read_pixmap_from = READ_FROM_STRING;
    cp->data_filename = argv[1];
  } else {
    cp->read_data_from = READ_FROM_STRING;
    cp->read_pixmap_from = READ_FROM_STRING;
    cp->data_filename = NULL;
  }
}

/*****************************************************************************/
/* ǡեɤ߹                                                  */
/*****************************************************************************/

void SetDefaultEyeValue(Clock * cp, Eye ** epp)
{
  (**epp).point.x  = (**epp).point.y  = 50;
  (**epp).center.x = (**epp).center.y = 0;
  (**epp).move.x = (**epp).move.y = 100;
  (**epp).radius.x = (**epp).radius.y = 8;
  strcpy((**epp).eye_foreground_color, cp->default_eye_foreground_color);
  strcpy((**epp).eye_background_color, cp->default_eye_background_color);
  (**epp).number = 0;
  (**epp).figure = NULL;
}

void SetNewEye(Clock * cp, Eye ** epp)
{
  Eye * eye;

  eye = *epp;
  *epp = malloc(sizeof(Eye));
  (**epp).next = eye;
  SetDefaultEyeValue(cp, epp);
}

XPoint ReadXPoint(void * p, int type)
{
  XPoint point;
  point.x = ReadInt(p, type);
  point.y = ReadInt(p, type);
  return (point);
}

void ReadDataFile(EyeClockWindow * wp, Clock * cp, Eye ** epp)
{
  char string[128];
  char filename[256];
  int i, t;
  void * p;

  if (cp->read_data_from == READ_FROM_FILE) {
    t = READ_FROM_FILE;
    strcpy(filename, cp->data_filename);
    if ((p = (void *)fopen(filename, "rt")) == NULL) {
      strcpy(filename, EYECLOCK_LIB);
      strcat(filename, cp->data_filename);
      if ((p = (void *)fopen(filename, "rt")) == NULL) {
	t = READ_FROM_STRING;
	p = (void *)DEFAULT_DATA;
        StringPointer(CLEAR);
      }
    }
  } else if (cp->read_data_from == READ_FROM_STRING) {
    t = READ_FROM_STRING;
    p = (void *)DEFAULT_DATA;
    StringPointer(CLEAR);
  } else if (cp->read_data_from == NOT_READ) return;

  while (ReadString(p, t, string) != EOF) {
    if (!strcmp(string, "Pixmap")) {
      ReadString(p, t, cp->pixmap_filename);
      cp->read_pixmap_from = READ_FROM_FILE;
    }
    else if (!strcmp(string, "Name")) ReadString(p, t, cp->clock_name);
    else if (!strcmp(string, "Width" )) cp->width = ReadInt(p, t);
    else if (!strcmp(string, "Height")) cp->height = ReadInt(p, t);
    else if (!strcmp(string, "ClockFlag")) cp->clock_flag = ReadFlag(p, t);
    else if (!strcmp(string, "EyeFlag")) cp->eye_flag = ReadFlag(p, t);
    else if (!strcmp(string, "EveryFlag")) cp->every_flag = ReadFlag(p, t);
    else if (!strcmp(string, "DefaultEyeForegroundColor"))
      ReadString(p, t, cp->default_eye_foreground_color);
    else if (!strcmp(string, "DefaultEyeBackgroundColor"))
      ReadString(p, t, cp->default_eye_background_color);
    else if (!strcmp(string, "LongForegroundColor"))
      ReadString(p, t, cp->long_foreground_color);
    else if (!strcmp(string, "LongBackgroundColor"))
      ReadString(p, t, cp->long_background_color);
    else if (!strcmp(string, "ShortForegroundColor"))
      ReadString(p, t, cp->short_foreground_color);
    else if (!strcmp(string, "ShortBackgroundColor"))
      ReadString(p, t, cp->short_background_color);
    else if (!strcmp(string, "LongLength")) cp->long_length = ReadInt(p, t);
    else if (!strcmp(string, "LongForegroundThickness"))
      cp->long_foreground_thickness = ReadInt(p, t);
    else if (!strcmp(string, "LongBackgroundThickness"))
      cp->long_background_thickness = ReadInt(p, t);
    else if (!strcmp(string, "ShortLength")) cp->short_length = ReadInt(p, t);
    else if (!strcmp(string, "ShortForegroundThickness"))
      cp->short_foreground_thickness = ReadInt(p, t);
    else if (!strcmp(string, "ShortBackgroundThickness"))
      cp->short_background_thickness = ReadInt(p, t);
    else if (!strcmp(string, "Center")) cp->clock_center = ReadXPoint(p, t);
    else if (!strcmp(string, "NewEye")) SetNewEye(cp, epp);
    else if (!strcmp(string, "EyePoint")) (**epp).point = ReadXPoint(p, t);
    else if (!strcmp(string, "EyeCenter")) (**epp).center = ReadXPoint(p, t);
    else if (!strcmp(string, "EyeMove")) (**epp).move = ReadXPoint(p, t);
    else if (!strcmp(string, "EyeRadius")) (**epp).radius = ReadXPoint(p, t);
    else if (!strcmp(string, "EyeForegroundColor"))
      ReadString(p, t, (**epp).eye_foreground_color);
    else if (!strcmp(string, "EyeBackgroundColor"))
      ReadString(p, t, (**epp).eye_background_color);
    else if (!strcmp(string, "EyeFigureNumber")) {
      (**epp).number = ReadInt(p, t);
      (**epp).figure = malloc(sizeof(XPoint) * (**epp).number);
    } else if (!strcmp(string, "EyeFigure"))
      for (i = 0; i < (**epp).number; i++)
	(**epp).figure[i] = ReadXPoint(p, t);
    else fprintf(stderr, "Not defined parameter: %s\n", string);
  }
  if (t == READ_FROM_FILE) fclose(p);
}

/*****************************************************************************/
/* ޥɥ饤ɤߤ                                                  */
/*****************************************************************************/

void Help()
{
  printf("%s", help_string);
  exit (0);
}

void ReadCommandParameter(Clock * cp, Eye * ep, int c, char ** v)
{
  int i;
  Eye * eye;

  if (cp->read_data_from == READ_FROM_FILE) i = 2; else i = 1;
  for (; i < c; i++) {
    if      (!strcmp(v[i], "-h"         )) Help();
    else if (!strcmp(v[i], "-help"      )) Help();
    else if (!strcmp(v[i], "-clock"     )) cp->clock_flag = ON;
    else if (!strcmp(v[i], "-noclock"   )) cp->clock_flag = OFF;
    else if (!strcmp(v[i], "-eye"       )) cp->eye_flag   = ON;
    else if (!strcmp(v[i], "-noeye"     )) cp->eye_flag   = OFF;
    else if (!strcmp(v[i], "-every"     )) cp->every_flag = ON;
    else if (!strcmp(v[i], "-noevery"   )) cp->every_flag = OFF;
    else if (!strcmp(v[i], "-v"         )) cp->verbose_flag = ON;
    else if (!strcmp(v[i], "-verbose"   )) cp->verbose_flag = ON;
    else if (!strcmp(v[i], "-noverbose" )) cp->verbose_flag = OFF;
    else if (i < c - 1) {
      if (!strcmp(v[i], "-pixmap")) {
        strcpy(cp->pixmap_filename, v[++i]);
        cp->read_pixmap_from = READ_FROM_FILE;
      }
      else if (!strcmp(v[i], "-name")) strcpy(cp->clock_name, v[++i]);
      else if (!strcmp(v[i], "-width" )) cp->width  = atoi(v[++i]);
      else if (!strcmp(v[i], "-height")) cp->height = atoi(v[++i]);
      else if ( (!strcmp(v[i],   "-eye-foreground-color")) ||
		(!strcmp(v[i],   "-eye-fg-color"        )) ) {
	strcpy(cp->default_eye_foreground_color, v[++i]);
	eye = ep;
	while (eye != NULL) {
	  strcpy(eye->eye_foreground_color, cp->default_eye_foreground_color);
	  eye = eye->next;
	}
      }
      else if ( (!strcmp(v[i],   "-eye-background-color")) ||
		(!strcmp(v[i],   "-eye-bg-color"        )) ) {
	strcpy(cp->default_eye_background_color, v[++i]);
	eye = ep;
	while (eye != NULL) {
	  strcpy(eye->eye_background_color, cp->default_eye_background_color);
	  eye = eye->next;
	}
      }
      else if ( (!strcmp(v[i],  "-long-foreground-color")) ||
		(!strcmp(v[i],  "-long-fg-color"        )) )
	strcpy( cp->long_foreground_color, v[++i]);
      else if ( (!strcmp(v[i],  "-long-background-color")) ||
		(!strcmp(v[i],  "-long-bg-color"        )) )
	strcpy( cp->long_background_color, v[++i]);
      else if ( (!strcmp(v[i], "-short-foreground-color")) ||
		(!strcmp(v[i], "-short-fg-color"        )) )
	strcpy(cp->short_foreground_color, v[++i]);
      else if ( (!strcmp(v[i], "-short-background-color")) ||
		(!strcmp(v[i], "-short-bg-color"        )) )
	strcpy(cp->short_background_color, v[++i]);
      else if (!strcmp(v[i], "-long-length" )) cp->long_length = atoi(v[++i]);
      else if ( (!strcmp(v[i], "-long-foreground-thickness" )) ||
		(!strcmp(v[i], "-long-fg-thickness"         )) )
	cp->long_foreground_thickness = atoi(v[++i]);
      else if ( (!strcmp(v[i], "-long-background-thickness" )) ||
		(!strcmp(v[i], "-long-bg-thickness" )) )
	cp->long_background_thickness = atoi(v[++i]);
      else if (!strcmp(v[i], "-short-length")) cp->short_length = atoi(v[++i]);
      else if ( (!strcmp(v[i], "-short-foreground-thickness" )) ||
		(!strcmp(v[i], "-short-fg-thickness"         )) )
	cp->short_foreground_thickness = atoi(v[++i]);
      else if ( (!strcmp(v[i], "-short-background-thickness" )) ||
		(!strcmp(v[i], "-short-bg-thickness" )) )
	cp->short_background_thickness = atoi(v[++i]);
      else if (i < c - 2) {
	if (!strcmp(v[i], "-center" )) {
	  cp->clock_center.x = atoi(v[++i]);
	  cp->clock_center.y = atoi(v[++i]);
	}
      }
    }
  }
}

/*****************************************************************************/
/* ѥ᡼ͤɽ                                                        */
/*****************************************************************************/

void CheckData(Clock * cp, Eye * ep)
{
  int i;

  printf("Pixmap = %s\n", cp->pixmap_filename);
  printf("Name = %s\n", cp->clock_name);
  printf("Width = %d\n", cp->width);
  printf("Height = %d\n", cp->height);
  printf("ClockFlag = %d\n", cp->clock_flag);
  printf("EyeFlag = %d\n", cp->eye_flag);
  printf("EveryFlag = %d\n", cp->every_flag);
  printf("DefaultEyeForegroundColor = %s\n", cp->default_eye_foreground_color);
  printf("DefaultEyeBackgroundColor = %s\n", cp->default_eye_background_color);
  printf("LongForegroundColor = %s\n", cp->long_foreground_color);
  printf("LongBackgroundColor = %s\n", cp->long_background_color);
  printf("ShortForegroundColor = %s\n", cp->short_foreground_color);
  printf("ShortBackgroundColor = %s\n", cp->short_background_color);
  printf("LongLength = %d\n", cp->long_length);
  printf("LongForegroundThickness = %d\n", cp->long_foreground_thickness);
  printf("LongBackgroundThickness = %d\n", cp->long_background_thickness);
  printf("ShortLength = %d\n", cp->short_length);
  printf("ShortForegroundThickness = %d\n", cp->short_foreground_thickness);
  printf("ShortBackgroundThickness = %d\n", cp->short_background_thickness);
  printf("Center = %d %d\n", cp->clock_center.x, cp->clock_center.y);
  while (ep != NULL) {
    printf("EyePoint = %d %d\n", ep->point.x, ep->point.y);
    printf("EyeCenter = %d %d\n", ep->center.x, ep->center.y);
    printf("EyeMove = %d %d\n", ep->move.x, ep->move.y);
    printf("EyeRadius = %d %d\n", ep->radius.x, ep->radius.y);
    printf("EyeForegroundColor = %s\n", ep->eye_foreground_color);
    printf("EyeBackgroundColor = %s\n", ep->eye_background_color);
    printf("EyeFigureNumber = %d\n", ep->number);
    printf("EyeFigure = ");
    for (i = 0; i < ep->number; i++)
      printf("(%d, %d), ", ep->figure[i].x, ep->figure[i].y);
    printf("\n");
    ep = ep->next;
  }
  fflush(stdout);
}

/*****************************************************************************/
/* åȤȥɥκ                                            */
/*****************************************************************************/

void CreateWindow(EyeClockWindow * wp, Clock * cp, int argc, char ** argv)
{
  int n;
  Arg args[10];

  n = 0;
  XtSetArg(args[n], XtNwidth,  cp->width ); n++;
  XtSetArg(args[n], XtNheight, cp->height); n++;
  wp->toplevel = XtAppInitialize(&(wp->app_context), "EyeClock",
				 NULL, 0, &argc, argv, NULL, args, n);
  XtRealizeWidget(wp->toplevel);

  wp->display = XtDisplay(wp->toplevel);
  wp->toplevel_window = XtWindow(wp->toplevel);
  wp->clock_window =
    XCreateSimpleWindow(wp->display, wp->toplevel_window,
			0, 0, cp->width, cp->height, 0,
			BlackPixel(wp->display, DefaultScreen(wp->display)),
			BlackPixel(wp->display, DefaultScreen(wp->display)));
/*  wp->clock_window = wp->toplevel_window;*/
  wp->root_window = DefaultRootWindow(wp->display);
  XMapWindow(wp->display, wp->clock_window);
}

/*****************************************************************************/
/* ԥåޥåפκ                                                      */
/*****************************************************************************/

void CreateClockPixmap(EyeClockWindow * wp, Clock * cp)
{
  XWindowAttributes attributes;
  char filename[256];
  FILE * fp;

  cp->depth = DefaultDepth(wp->display, DefaultScreen(wp->display));

  if (cp->read_pixmap_from == READ_FROM_FILE) {
    strcpy(filename, cp->pixmap_filename);
    if ((fp = fopen(filename, "rt")) == NULL) {
      strcpy(filename, EYECLOCK_LIB);
      strcat(filename, cp->pixmap_filename);
      if ((fp = fopen(filename, "rt")) == NULL)
        wp->back_pixmap = MakePixmapFromData(wp->display, wp->clock_window,
					     DEFAULT_PIXMAP,
					     &(cp->width), &(cp->height));
      else {
	fclose(fp);
        wp->back_pixmap = MakePixmapFromFile(wp->display, wp->clock_window,
					     filename,
					     &(cp->width), &(cp->height));
      }
    } else {
      fclose(fp);
      wp->back_pixmap = MakePixmapFromFile(wp->display, wp->clock_window,
					   filename,
					   &(cp->width), &(cp->height));
    }
  } else if (cp->read_pixmap_from == READ_FROM_STRING)
    wp->back_pixmap = MakePixmapFromData(wp->display, wp->clock_window,
					 DEFAULT_PIXMAP,
					 &(cp->width), &(cp->height));
  else if (cp->read_pixmap_from == NOT_READ)
    wp->back_pixmap  = XCreatePixmap(wp->display, wp->clock_window,
				     cp->width, cp->height, cp->depth);

  XResizeWindow(wp->display, wp->toplevel_window, cp->width, cp->height);
  XResizeWindow(wp->display, wp->clock_window, cp->width, cp->height);
  wp->clock_pixmap  = XCreatePixmap(wp->display, wp->clock_window,
				    cp->width, cp->height, cp->depth);
  wp->figure_pixmap = XCreatePixmap(wp->display, wp->clock_window,
				    cp->width, cp->height, cp->depth);
  wp->eye_pixmap    = XCreatePixmap(wp->display, wp->clock_window,
				    cp->width, cp->height, cp->depth);
  wp->work_pixmap   = XCreatePixmap(wp->display, wp->clock_window,
				    cp->width, cp->height, cp->depth);
}

/*****************************************************************************/
/* פͤ                                                            */
/*****************************************************************************/

void SetClockValue(Clock * cp)
{
  if (cp->clock_center.x == DEFAULT_CLOCK_CENTER_X)
    cp->clock_center.x = cp->width /2;
  if (cp->clock_center.y == DEFAULT_CLOCK_CENTER_Y)
    cp->clock_center.y = cp->height/2;
  if (cp->long_length == DEFAULT_LONG_LENGTH)
    if (cp->width > cp->height) cp->long_length = (cp->height / 2) - 10;
    else cp->long_length = (cp->width / 2) - 10;
  if (cp->short_length == DEFAULT_SHORT_LENGTH)
    cp->short_length = (cp->long_length / 2);
  if (cp->long_foreground_thickness == DEFAULT_LONG_FOREGROUND_THICKNESS)
    cp->long_foreground_thickness = cp->long_length / 10 + 2;
  if (cp->long_background_thickness == DEFAULT_LONG_BACKGROUND_THICKNESS)
    cp->long_background_thickness = cp->long_foreground_thickness + 6;
  if (cp->short_foreground_thickness == DEFAULT_SHORT_FOREGROUND_THICKNESS)
    cp->short_foreground_thickness = cp->long_foreground_thickness + 8;
  if (cp->short_background_thickness == DEFAULT_SHORT_BACKGROUND_THICKNESS)
    cp->short_background_thickness = cp->long_foreground_thickness + 12;
}

/*****************************************************************************/
/* GC κ                                                                 */
/*****************************************************************************/

void SetClockGC(EyeClockWindow * wp, Clock * cp)
{
  Display * d;
  Window w;

  d = wp->display;
  w = wp->clock_window;
  wp->gc_long        = XCreateGC(d, w, 0, 0);
  wp->gc_long_back   = XCreateGC(d, w, 0, 0);
  wp->gc_long_clear  = XCreateGC(d, w, 0, 0);
  wp->gc_short       = XCreateGC(d, w, 0, 0);
  wp->gc_short_back  = XCreateGC(d, w, 0, 0);
  wp->gc_short_clear = XCreateGC(d, w, 0, 0);
  XSetForeground(d, wp->gc_long      ,GetColor(d, cp->long_foreground_color));
  XSetForeground(d, wp->gc_long_back ,GetColor(d, cp->long_background_color));
  XSetForeground(d, wp->gc_long_clear,BlackPixel(d, DefaultScreen(d)));
  XSetForeground(d, wp->gc_short     ,GetColor(d,cp->short_foreground_color));
  XSetForeground(d, wp->gc_short_back,GetColor(d,cp->short_background_color));
  XSetForeground(d, wp->gc_short_clear,BlackPixel(d, DefaultScreen(d)));
  XSetLineAttributes(d, wp->gc_long,         cp->long_foreground_thickness,
		     LineSolid,CapRound,JoinMiter);
  XSetLineAttributes(d, wp->gc_long_back,    cp->long_background_thickness,
		     LineSolid,CapRound,JoinMiter);
  XSetLineAttributes(d, wp->gc_long_clear,   cp->long_background_thickness,
		     LineSolid,CapRound,JoinMiter);
  XSetLineAttributes(d, wp->gc_short,       cp->short_foreground_thickness,
		     LineSolid,CapRound,JoinMiter);
  XSetLineAttributes(d, wp->gc_short_back,  cp->short_background_thickness,
		     LineSolid,CapRound,JoinMiter);
  XSetLineAttributes(d, wp->gc_short_clear, cp->short_background_thickness,
		     LineSolid,CapRound,JoinMiter);
  wp->gc_copy_copy = XCreateGC(d, w, 0, 0);
  wp->gc_copy_and  = XCreateGC(d, w, 0, 0);
  wp->gc_copy_andinv  = XCreateGC(d, w, 0, 0);
  wp->gc_copy_or   = XCreateGC(d, w, 0, 0);
  XSetFunction(d, wp->gc_copy_copy, GXcopy);
  XSetFunction(d, wp->gc_copy_and, GXand);
  XSetFunction(d, wp->gc_copy_andinv, GXandInverted);
  XSetFunction(d, wp->gc_copy_or, GXor);
}

void SetEyeGC(EyeClockWindow * wp, Eye * ep)
{
  Display * d;
  Window w;

  d = wp->display;
  w = wp->clock_window;
  wp->gc_eye_black      = XCreateGC(d, w, 0, 0);
  wp->gc_eye_white      = XCreateGC(d, w, 0, 0);
  XSetForeground(d, wp->gc_eye_black, BlackPixel(d, DefaultScreen(d)));
  XSetBackground(d, wp->gc_eye_black, BlackPixel(d, DefaultScreen(d)));
  XSetForeground(d, wp->gc_eye_white, WhitePixel(d, DefaultScreen(d)));
  XSetBackground(d, wp->gc_eye_white, WhitePixel(d, DefaultScreen(d)));
  XSetFillRule(  d, wp->gc_eye_black, EvenOddRule);
  XSetFillRule(  d, wp->gc_eye_white, EvenOddRule);
  XSetArcMode(   d, wp->gc_eye_black, ArcPieSlice);
  XSetArcMode(   d, wp->gc_eye_white, ArcPieSlice);
  while (ep != NULL) {
    ep->gc_eye_foreground = XCreateGC(d, w, 0, 0);
    ep->gc_eye_background = XCreateGC(d, w, 0, 0);
    XSetForeground(d, ep->gc_eye_foreground,
		   GetColor(d, ep->eye_foreground_color));
    XSetBackground(d, ep->gc_eye_foreground, BlackPixel(d, DefaultScreen(d)));
    XSetForeground(d, ep->gc_eye_background,
		   GetColor(d, ep->eye_background_color));
    XSetBackground(d, ep->gc_eye_background, BlackPixel(d, DefaultScreen(d)));
    XSetFillRule(  d, ep->gc_eye_foreground, EvenOddRule);
    XSetFillRule(  d, ep->gc_eye_background, EvenOddRule);
    XSetArcMode(   d, ep->gc_eye_foreground, ArcPieSlice);
    XSetArcMode(   d, ep->gc_eye_background, ArcPieSlice);
    ep = ep->next;
  }
}

/*****************************************************************************/
/* ԥåޥåפ                                                      */
/*****************************************************************************/

void SetEyePixmap(EyeClockWindow * wp, Clock * cp, Eye * ep)
{
  int i;

  XFillRectangle(wp->display, wp->figure_pixmap, wp->gc_eye_white, 0, 0,
		 cp->width, cp->height);
  XFillRectangle(wp->display, wp->eye_pixmap, wp->gc_eye_black, 0, 0,
		 cp->width, cp->height);
  while (ep != NULL) {
    for (i = 0; i < ep->number; i++) {
      ep->figure[i].x += ep->point.x;
      ep->figure[i].y += ep->point.y;
    }
    XFillPolygon(wp->display, wp->figure_pixmap, wp->gc_eye_black,
		 ep->figure, ep->number, Complex, CoordModeOrigin);
    XFillPolygon(wp->display, wp->eye_pixmap, ep->gc_eye_background,
		 ep->figure, ep->number, Complex, CoordModeOrigin);
    ep = ep->next;
  }
}

/*****************************************************************************/
/*                                                             */
/*****************************************************************************/

void SetIcon(EyeClockWindow * wp)
{
  XWMHints wmh;

  wp->icon_pixmap =
    MakePixmapFromData(wp->display, wp->root_window,
		       ICON, &(wp->icon_width), &(wp->icon_height));
  wmh.flags = IconPixmapHint;
  wmh.icon_pixmap = wp->icon_pixmap;
  XSetWMHints(wp->display, wp->toplevel_window, &wmh);
}

/*****************************************************************************/
/* פν                                                              */
/*****************************************************************************/

void SetWindowAttributes(EyeClockWindow * wp, Clock * cp)
{
  XSizeHints sh;
  XWMHints wmhints;

  if (cp->eye_flag == ON)
    XSelectInput(wp->display, wp->root_window, PointerMotionMask);
  XSelectInput(wp->display, wp->clock_window,
	       KeyPressMask | ExposureMask);
  XSelectInput(wp->display, wp->toplevel_window, StructureNotifyMask);

  sh.flags = (PBaseSize | PResizeInc | PMinSize | PMaxSize);
  sh.width_inc = sh.height_inc = 0;
  sh.base_width  = sh.min_width  = sh.max_width  = cp->width;
  sh.base_height = sh.min_height = sh.max_height = cp->height;
  XSetWMNormalHints(wp->display, wp->toplevel_window, &sh);

  wmhints.input = True;
  XSetWMHints(wp->display, wp->toplevel_window, &wmhints);

  wp->atom_1 = XInternAtom(wp->display, "WM_PROTOCOLS", False);
  wp->atom_2 = XInternAtom(wp->display, "WM_DELETE_WINDOW", False);
  XSetWMProtocols(wp->display, wp->toplevel_window, &(wp->atom_2), 1);

  XStoreName(wp->display, wp->toplevel_window, cp->clock_name);
  XSetIconName(wp->display, wp->toplevel_window, cp->clock_name);

  SetIcon(wp);
}

/*****************************************************************************/
/* End of File.                                                              */
/*****************************************************************************/
